SystemOrganization addCategory: #'PN-MorphIDE'!

ZnStaticFileServerDelegate subclass: #PNjtServerIDE
	instanceVariableNames: 'ideFilename jtFilename bootCategories newCategories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PN-MorphIDE'!

!PNjtServerIDE commentStamp: 'PierceNg 9/3/2011 17:18' prior: 0!
I am the MorphIDE server. My operation is rather tightly coupled with the way JTalk works.!

!PNjtServerIDE methodsFor: 'accessing' stamp: 'PierceNg 9/3/2011 18:50'!
bootCategories 
	^ bootCategories  ! !

!PNjtServerIDE methodsFor: 'private' stamp: 'PierceNg 9/3/2011 17:01'!
categoryName: aString
	
	"aString looks like this: /st/PN-Smalltalking.st"
	^ aString collect: [ :x | x ] from: 4 to: (aString size - 3) ! !

!PNjtServerIDE methodsFor: 'processing' stamp: 'PierceNg 9/3/2011 18:41'!
handlePut: aRequest
	| upath lfile |
	
	upath := aRequest uri path.
	lfile := self directory fullNameFor: upath.
	self directory deleteFileNamed: upath.
	NSFileStream fileNamed: lfile 
		forWriting: true
		do: [ :stream | 
			stream binary.
			stream nextPutAll: (aRequest entity contents) ].

	self unionFindCategory: upath.  

	^ ZnResponse created: (ZnUrl fromString: aRequest requestLine uri path).! !

!PNjtServerIDE methodsFor: 'processing' stamp: 'PierceNg 9/3/2011 18:33'!
handleRequest: aRequest

	(aRequest method = #PUT)
		ifTrue: [ ^ self handlePut: aRequest ]
		ifFalse: [ ^ super handleRequest: aRequest ]! !

!PNjtServerIDE methodsFor: 'accessing' stamp: 'PierceNg 9/2/2011 22:56'!
ideFilename 
	^ ideFilename ! !

!PNjtServerIDE methodsFor: 'private' stamp: 'PierceNg 9/2/2011 22:57'!
indexFileIn: aDir
	
	(aDir fileExists: self ideFilename) 
		ifTrue: [ ^ (aDir directoryEntryFor: self ideFilename) fullName ]
		ifFalse: [ ^ super indexFileIn: aDir ]! !

!PNjtServerIDE methodsFor: 'initializating' stamp: 'PierceNg 9/17/2011 20:44'!
initialize 

	ideFilename := 'morphIDE.html'.
	jtFilename := 'stcategories.txt'.
	bootCategories := Set new.
	newCategories := Set new.
! !

!PNjtServerIDE methodsFor: 'accessing' stamp: 'PierceNg 9/3/2011 17:23'!
jtFilename 
	^ jtFilename ! !

!PNjtServerIDE methodsFor: 'private' stamp: 'PierceNg 9/3/2011 18:51'!
loadJtalkBootCategories
	| stream |
	
	bootCategories := Set new.
	stream := NSFileStream fileNamed: (self directory fullNameFor: self jtFilename)
		forWriting: false.
	[ stream atEnd ]
		whileFalse: [ bootCategories add: stream nextLine ].
	stream close.! !

!PNjtServerIDE methodsFor: 'private-morphing' stamp: 'PierceNg 9/17/2011 20:54'!
morphide: aCollection
	| lfile |
	
	lfile := self directory fullNameFor: self ideFilename.
	self directory deleteFileNamed: self ideFilename ifAbsent: [].
	NSFileStream fileNamed: lfile
		forWriting: true
		do: [ :stream |
			stream textual.
			stream nextPutAll: self morphragBefore. 
			aCollection isEmpty ifFalse: [
				stream nextPutAll: '{files:['.
				aCollection allButLastDo: [ :ea | stream nextPutAll: '"', ea, '.js",' ].
				stream nextPutAll: '"', aCollection last, '.js"]}' ].
			stream nextPutAll: self morphragAfter ].
 		
	! !

!PNjtServerIDE methodsFor: 'private-morphing' stamp: 'PierceNg 9/17/2011 20:44'!
morphragAfter

^ ');
</script>

<div class="box">

<h1>MorphIDE - The Morphing Amber IDE</h1>

<p><button onclick="smalltalk.Browser._open()">Class Browser</button></p>

<p>This web page hosts the Amber IDE. It is intended to be served by MorphIDE,
which works as follows:</p>

<ul>

<li>MorphIDE is a static file HTTP server. It serves the Javascript files
that make up the Amber IDE. The IDE itself is initialized by the Javascript
function <code>loadAmber()</code>.</li>

<li>MorphIDE accepts HTTP PUTs from the Amber IDE, whereupon it will
(over)write the .js and .st files being PUT to filesystem. Note that
MorphIDE is <strong>not</strong> a WebDAV server.</li>

<li>Suppose you create a new category in Amber called "MyHacks", write some
code, and "commit category". First MorphIDE will save the My-Hacks .js and
.st files that Amber PUTs. Then it will modify this web page; in
particular, it will change the <code>loadAmber()</code> call to
<code>loadAmber(["My-Hacks.js"])</code> instead. In this way, the next time
you load this web page again, your code in the "My-Hacks" category is
readily available.</li>

</ul>

<p>MorphIDE is written by Pierce Ng in Pharo Smalltalk, using Zinc HTTP. It
was written because it was more fun than messing with Apache config on my 
MBP. MorphIDE is easily extended to invoke Git, Fossil, etc. via 
OSProcess/CommandShell.</p>

</div>
</body>
</html>
'! !

!PNjtServerIDE methodsFor: 'private-morphing' stamp: 'PierceNg 9/17/2011 20:43'!
morphragBefore

^ '<!!DOCTYPE html>
<html>
<head>
<title>MorphIDE - The Morphing Amber IDE</title>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<meta name="author" content="Pierce Ng">
<link rel="stylesheet" type="text/css" href="./ide/style.css">
<script type="text/javascript" src="./js/amber.js"></script>
</head>

<body>
<script type="text/javascript">
loadAmber(
'! !

!PNjtServerIDE methodsFor: 'accessing' stamp: 'PierceNg 9/3/2011 18:48'!
newCategories 
	^ newCategories  ! !

!PNjtServerIDE methodsFor: 'private' stamp: 'PierceNg 9/3/2011 18:51'!
unionFindCategory: aString
	| category |
	
	(aString = self jtFilename) 
		ifTrue: [ self loadJtalkBootCategories ]
		ifFalse: [
			(aString endsWith: '.st') ifTrue: [ 
				category := self categoryName: aString.  
				((self newCategories includes: category) or: [ self bootCategories includes: category ])
					ifFalse: [ 
						self newCategories add: category.
						self morphide: self newCategories asOrderedCollection ]]]		! !

ZnSingleThreadedServer subclass: #PNjtPutServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PN-MorphIDE'!

!PNjtPutServer commentStamp: 'PierceNg 9/3/2011 17:13' prior: 0!
I am a specialist server that handles HTTP PUTs from the JTalk IDE.!

!PNjtPutServer methodsFor: 'private' stamp: 'PierceNg 9/1/2011 00:10'!
readRequest: stream
	| header request |

	request := ZnRequest readHeaderFrom: stream.
	(request method = #PUT)
		ifFalse: [ request readEntityFrom: stream ]
		ifTrue: [ 
			| entityReader |
			(entityReader := request entityReaderOn: stream) streaming.
			request entity: entityReader readEntity ].
	request headers 
		at: ZnConstants remoteAddressHeader
		put: (ZnNetworkingUtils ipAddressToString: stream socket remoteAddress).
	self logRequest: request.
	^ request! !
